package com.cf.cs.authentication.configure;

import com.cf.cs.authentication.security.impl.CustomAuthorizationManager;
import com.cf.cs.authentication.security.impl.JwtAccessDeniedHandler;
import com.cf.cs.authentication.security.impl.JwtAuthenticationEntryPoint;
import com.cf.cs.authentication.security.impl.JwtAuthenticationFilter;
import com.cf.cs.authentication.security.impl.JwtCustomUsernamePassword;
import com.cf.cs.authentication.security.impl.JwtLogoutHandler;
import com.cf.cs.authentication.security.impl.JwtLogoutSuccessHandler;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.security.web.authentication.logout.LogoutFilter;
import org.springframework.security.web.context.HttpSessionSecurityContextRepository;

/**
 * The type Security.
 *
 * @author xajhuang @163.com
 */
@Configuration
@EnableWebSecurity
@Slf4j
public class SecuritySecurity {
    /**
     * The User details service.
     */
    @Resource
    UserDetailsService userDetailsService;
    @Resource
    private JwtAuthenticationEntryPoint jwtAuthenticationEntryPoint;

    @Resource
    private JwtAccessDeniedHandler jwtAccessDeniedHandler;

    @Resource
    private JwtLogoutSuccessHandler jwtLogoutSuccessHandler;

    @Resource
    private JwtLogoutHandler jwtLogoutHandler;

    @Resource
    private JwtAuthenticationFilter tokenFilter;

    @Resource
    private CustomAuthorizationManager customAuthorizationManager;

    @Resource
    private UserSecurityConfigure userSecurityConfigure;

    /**
     * Password encoder password encoder.
     *
     * @return the password encoder
     */
    @Bean
    PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    /**
     * Filter chain security filter chain.
     *
     * @param http the http
     * @return the security filter chain
     * @throws Exception the exception
     */
    @Bean
    public SecurityFilterChain filterChain(HttpSecurity http) throws Exception {

        http.authorizeHttpRequests(resp -> {
                // 配置文件中配置的白名单
                userSecurityConfigure.getWhiteList().forEach(k -> resp.requestMatchers(k.getMethod(), k.getUrl()).permitAll());

                resp.requestMatchers("/api/**").access(customAuthorizationManager);
            })
            .sessionManagement(session -> session.sessionCreationPolicy(SessionCreationPolicy.STATELESS))
            .formLogin(AbstractHttpConfigurer::disable)
            .exceptionHandling(exceptions -> exceptions.authenticationEntryPoint(jwtAuthenticationEntryPoint)
                                                         .accessDeniedHandler(jwtAccessDeniedHandler))
            .logout(logout -> logout.logoutUrl("/api/logout")
                    .addLogoutHandler(jwtLogoutHandler)
                    .logoutSuccessHandler(jwtLogoutSuccessHandler)
                    .clearAuthentication(true)
                   )
            .csrf(AbstractHttpConfigurer::disable);

        http.addFilterBefore(tokenFilter, UsernamePasswordAuthenticationFilter.class);
        http.addFilterBefore(tokenFilter, LogoutFilter.class);
        http.addFilterAt(jwtCustomUsernamePassword(), UsernamePasswordAuthenticationFilter.class);
        return http.build();
    }

    /**
     * Jwt custom username password jwt custom username password.
     *
     * @return the jwt custom username password
     */
    @Bean
    public JwtCustomUsernamePassword jwtCustomUsernamePassword() {
        JwtCustomUsernamePassword jwtCustomUsernamePassword = new JwtCustomUsernamePassword();
        jwtCustomUsernamePassword.setAuthenticationManager(authenticationManager());
        jwtCustomUsernamePassword.setFilterProcessesUrl("/api/login");
        jwtCustomUsernamePassword.setSecurityContextRepository(new HttpSessionSecurityContextRepository());
        return jwtCustomUsernamePassword;
    }

    /**
     * Authentication manager.
     *
     * @return the authentication manager
     */
    @Bean
    AuthenticationManager authenticationManager() {
        DaoAuthenticationProvider daoAuthenticationProvider = new DaoAuthenticationProvider();
        daoAuthenticationProvider.setUserDetailsService(userDetailsService);
        daoAuthenticationProvider.setPasswordEncoder(passwordEncoder());
        return new ProviderManager(daoAuthenticationProvider);
    }
}
