package com.cf.cs.authentication.misc;


import com.cf.cs.authentication.exception.CommonAuthException;
import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.config.JwtConfigure;
import com.cf.cs.crypto.arithmetic.CryptoHelper;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.service.UserDataBaseService;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.MalformedJwtException;
import io.jsonwebtoken.MissingClaimException;
import io.jsonwebtoken.security.Keys;
import io.micrometer.common.util.StringUtils;
import jakarta.annotation.Resource;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.security.NoSuchAlgorithmException;
import java.util.Date;

/**
 * The type Jwt utils.
 *
 * @author xajhuang @163.com
 */
@Data
@Component
@Slf4j
public class JwtUtils {
    @Resource
    private JwtConfigure jwtConfigure;

    @Resource
    private UserDataBaseService userDbService;

    /**
     * Gets head value.
     *
     * @return the head value
     */
    public String getHeadValue() {
        return jwtConfigure.getHttpHead();
    }

    /**
     * Gets token.
     *
     * @param uId the u id
     * @return the token
     */
    public String getToken(String uId) {
        Date nowDate    = new Date();
        Date expireDate = new Date(nowDate.getTime() + 1000 * jwtConfigure.getExpireTime());

        return Jwts.builder()
                   .header()
                   .add("typ", "JWT")
                   .and()
                   .signWith(Keys.hmacShaKeyFor(getSecretKey(jwtConfigure.getSecretKey())))
                   .expiration(expireDate)
                   .issuedAt(nowDate)
                   .id(uId)
                   .compact();
    }

    /**
     * Parse token user.
     *
     * @param jwt the jwt
     * @return the user
     */
    public User parseToken(String jwt) {
        try {

            if (!jwt.startsWith(ConstValue.STRING_HTTP_AUTH_HEAD)) {
                throw new CommonAuthException(ErrorCode.ERR_MISSAUTHHEAD);
            }

            jwt = jwt.replace(ConstValue.STRING_HTTP_AUTH_HEAD, "");
            Claims claim = Jwts.parser()
                               .verifyWith(Keys.hmacShaKeyFor(getSecretKey(jwtConfigure.getSecretKey())))
                               .build()
                               .parseSignedClaims(jwt)
                               .getPayload();

            if (isTokenExpired(claim)) {
                throw new CommonAuthException(ErrorCode.ERR_TOKENTIMEOUT);
            }

            String uid = claim.getId();

            if (StringUtils.isBlank(uid) || StringUtils.isEmpty(uid)) {
                throw new CommonAuthException(ErrorCode.ERR_TOKENNOTFOUND);
            }

            // 获取用户的权限等信息
            User user = userDbService.getAccountUserByUid(uid);

            if (user == null) {
                throw new CommonAuthException(ErrorCode.ERR_TOKENNOTFOUND);
            }

            return user;
        } catch (MalformedJwtException e) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY, "密钥算法或者密钥转换错误");
        } catch (MissingClaimException e) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY, "密钥缺少校验数据");
        } catch (ExpiredJwtException e) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY, "密钥已过期");
        } catch (JwtException e) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY, "密钥解析错误");
        }
    }

    private byte[] getSecretKey(String base64Key) {
        if (StringUtils.isBlank(base64Key)) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY);
        }

        try {
            return CryptoHelper.sha256Encryption(base64Key);
        } catch (NoSuchAlgorithmException e) {
            throw new CommonAuthException(ErrorCode.ERR_TOKEN_KEY);
        }
    }

    /**
     * Is token expired boolean.
     *
     * @param claims the claims
     * @return the boolean
     */
    public boolean isTokenExpired(Claims claims) {
        return claims.getExpiration().before(new Date());
    }
}
