package com.cf.cs.authentication.security.impl;

import com.cf.cs.authentication.exception.CommonAuthException;
import com.cf.cs.authentication.misc.JwtUtils;
import com.cf.cs.authentication.misc.SecurityResponseUtils;
import com.cf.cs.authentication.pojo.po.AuthAccountUser;
import com.cf.cs.authentication.service.AccountJwtCacheService;
import com.cf.cs.authentication.service.AuthUsersService;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.JwtCache;
import com.cf.cs.database.service.OperationLogDataBaseService;
import com.cf.cs.protocol.config.ProtocolConfigure;
import com.cf.cs.protocol.misc.ProtocolJsonUtils;
import com.cf.cs.protocol.pojo.dto.LoginReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.vo.LoginResp;
import com.cf.cs.protocol.validation.group.ValidGroups;
import com.fasterxml.jackson.core.type.TypeReference;
import jakarta.annotation.Resource;
import jakarta.servlet.FilterChain;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.AccountExpiredException;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.CredentialsExpiredException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.event.InteractiveAuthenticationSuccessEvent;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.bind.annotation.RequestMethod;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * The type Jwt custom username password.
 *
 * @author xajhuang @163.com
 */
public class JwtCustomUsernamePassword extends UsernamePasswordAuthenticationFilter {
    @Resource
    private AuthUsersService usersService;

    @Resource
    private JwtUtils jwtUtils;

    @Resource
    private AccountJwtCacheService accountJwtCacheService;

    @Resource
    private OperationLogDataBaseService optLogDbService;

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request,
                                                HttpServletResponse response) throws AuthenticationException {
        //获取请求头，据此判断请求参数类型
        String contentType = request.getContentType();
        if (MediaType.APPLICATION_JSON_VALUE.startsWith(contentType)) {
            //说明请求参数是 JSON
            if (!RequestMethod.POST.toString().equals(request.getMethod())) {
                throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
            }

            try {
                ProtocolReq<LoginReq> loginReq = ProtocolJsonUtils.jsonGetObject(request.getInputStream(),
                                                                                 new TypeReference<>() {
                                                                           });
                List<String> validate = HelperUtils.validate(loginReq, ValidGroups.ProtocolCommonValid.class,
                                                             ValidGroups.LogoutReqValid.class);
                // 如果校验通过，validate为空；否则，validate包含未校验通过项
                if (validate.isEmpty()) {
                    // 校验通过，才会执行业务逻辑处理
                    LoginReq us = loginReq.getMsgContent();

                    //构建登录令牌
                    UsernamePasswordAuthenticationToken authRequest =
                            UsernamePasswordAuthenticationToken.unauthenticated(
                                    us.getUsername(),
                                    us.getPassword());
                    // Allow subclasses to set the "details" property
                    setDetails(request, authRequest);
                    //执行真正的登录操作
                    return this.getAuthenticationManager().authenticate(authRequest);
                } else {
                    throw new CommonAuthException(ErrorCode.ERR_PARAMEXCEPTION, validate.toArray(new String[0]));
                }
            } catch (Exception exception) {
                if (exception instanceof IOException) {
                    throw new CommonAuthException(ErrorCode.ERR_JSON_DECODE);
                } else if (exception instanceof CommonAuthException e) {
                    throw new CommonAuthException(e.getErr(), e.getDescription());
                } else {
                    throw new AuthenticationServiceException(exception.getMessage(), exception.getCause());
                }
            }
        } else {
            return super.attemptAuthentication(request, response);
        }
    }

    @Override
    public void unsuccessfulAuthentication(HttpServletRequest request,
                                           HttpServletResponse response,
                                           AuthenticationException failed) throws IOException {
        ErrorCode      err = ErrorCode.ERR_SYSTEMEXCEPTION;
        BaseRespStatus rsp = new BaseRespStatus();

        SecurityContextHolder.clearContext();

        if (failed instanceof LockedException) {
            err = ErrorCode.ERR_USERLOCK;
        } else if (failed instanceof CredentialsExpiredException) {
            err = ErrorCode.ERR_PASSWORD_EXPIRED;
        } else if (failed instanceof AccountExpiredException) {
            err = ErrorCode.ERR_ACCOUNT;
        } else if (failed instanceof DisabledException) {
            err = ErrorCode.ERR_ACCOUNT;
        } else if (failed instanceof BadCredentialsException) {
            err = ErrorCode.ERR_PASSWORD;
        }

        if (failed instanceof AuthenticationServiceException) {
            if (failed.getCause() instanceof CommonAuthException ex) {
                rsp.setStatus(ex.getErr().getCode());
            } else {
                rsp.setStatus(ErrorCode.ERR_ACCOUNT.getCode());
            }
            rsp.setMessage(new String[] {failed.getMessage()});
        } else if (failed instanceof CommonAuthException ex) {
            rsp.setStatus(ex.getErr().getCode());
            rsp.setMessage(ex.getDescription());
        } else {
            rsp.setStatus(err.getCode());
            rsp.setMessage(new String[] {err.getDescription()});
        }

        SecurityResponseUtils.authenticationResponse(request, response, rsp, Arrays.toString(rsp.getMessage()),
                                                     optLogDbService);
    }

    @Override
    public void successfulAuthentication(HttpServletRequest request,
                                         HttpServletResponse response,
                                         FilterChain chain,
                                         Authentication authResult) throws IOException {
        AuthAccountUser au = usersService.getAuthAccountByUserName(authResult.getName());
        JwtCache        jc = accountJwtCacheService.getAccountJwtCache(au.getUid());

        if (jc != null) {
            accountJwtCacheService.deleteAccountJwtCache(au.getUid());
        }

        String jwt = jwtUtils.getToken(au.getUid());
        jc = accountJwtCacheService.createAccountJwtCache(au.getUid(), jwt);

        LoginResp loginRsp = LoginResp.builder()
                                      .username(authResult.getName())
                                      .token(jc.getJwt())
                                      .logTime(System.currentTimeMillis())
                                      .expireTime(System.currentTimeMillis() + ProtocolConfigure.TIMEOUT_OF_SECONDS * 1000L)
                                      .build();
        loginRsp.setMessage(new String[] {ErrorCode.ERR_OK.getDescription()});
        loginRsp.setStatus(ErrorCode.ERR_OK.getCode());

        SecurityContextHolder.getContext().setAuthentication(authResult);
        // Fire event
        if (this.eventPublisher != null) {
            eventPublisher.publishEvent(new InteractiveAuthenticationSuccessEvent(authResult, this.getClass()));
        }

        // 缓存 JWT 以及用户信息
        ApiContextUtils.setJwtUserInfo(jwt, au.getUsername(), au.getUid(), au.getId());

        try {
            SecurityResponseUtils.authenticationResponse(request, response, loginRsp,
                                                         "[" + authResult.getName() + "] login", optLogDbService);
        } finally {
            ApiContextUtils.clear();
        }

    }
}
