package com.cf.cs.database.service.impl;

import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.config.CommonConfigure;
import com.cf.cs.base.exception.CommonErrorCodeException;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.ApiContext;
import com.cf.cs.database.mapper.OperationLogMapper;
import com.cf.cs.database.pojo.entity.OperationLog;
import com.cf.cs.database.service.OperationLogDataBaseService;
import com.cf.cs.database.service.UserDataBaseService;
import com.jayway.jsonpath.JsonPath;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.spring.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.List;
import java.util.Optional;

import static com.cf.cs.database.pojo.entity.table.OperationLogTableDef.OPERATION_LOG;
import static com.cf.cs.database.pojo.entity.table.UserTableDef.USER;
import static com.mybatisflex.core.query.QueryMethods.distinct;
import static com.mybatisflex.core.query.QueryMethods.select;

/**
 * 系统操作日志 服务层实现。
 *
 * @author mybatis -flex-helper automatic generation
 * @since 1.0
 */
@Service
@Slf4j
public class OperationLogDataBaseServiceImpl extends ServiceImpl<OperationLogMapper, OperationLog> implements OperationLogDataBaseService {
    private static final int    MAX_DB_STRING = 4090;
    private static final String ERROR_URL     = "/error";

    @Resource
    private ServerProperties serverProperties;

    @Resource
    private UserDataBaseService userDbService;

    @Resource
    private OperationLogMapper operationLogMapper;

    @Override
    public void systemOperationLog(String call,
                                   HttpServletRequest request,
                                   Object result,
                                   OperationLogAnnotation annotation,
                                   ErrorCode errorCode) {
        if (annotation != null) {
            systemOperationLog(call, request, result, annotation.OperationModule(), annotation.OperationType(),
                               annotation.OperationDesc(), errorCode);
        } else {
            systemOperationLog(call, request, result, "", "", "", errorCode);
        }
    }

    @Override
    @SuppressWarnings("java:S3776")
    public void securityOperationLog(HttpServletRequest request, Object result, ErrorCode errorCode, String readme) {
        try {
            ApiContext   ctx    = ApiContextUtils.get();
            OperationLog optLog = new OperationLog();

            if (request.getRequestURI().startsWith(CommonConfigure.PROJECT_PREFIX_URL + ERROR_URL)) {
                return;
            }

            optLog.setOperationStatus(errorCode.getDescription());

            //获取注解操作信息
            optLog.setModule("系统安全模块");
            optLog.setOperationType("认证/鉴权");
            optLog.setDescription(readme);
            optLog.setUserId(-1L);

            //操作时间
            optLog.setOperationTime(new Timestamp(System.currentTimeMillis()));
            // 请求方法
            optLog.setHttpMethod(request.getMethod());
            // 请求URL
            String prefixUrl = Optional.ofNullable(serverProperties.getServlet().getContextPath()).orElse("");
            String url       = request.getRequestURI().replaceFirst("^" + prefixUrl, "");
            optLog.setHttpPath(url);

            // 仅处理 RESTful 接口
            String apiPrefix = "/api";
            if (url.startsWith(apiPrefix)) {
                // 登录接口不包含 Authorization 信息
                String loginUrl = "login";
                if (url.lastIndexOf(loginUrl) != -1) {
                    optLog.setCallFunction(loginUrl);
                    optLog.setUserId(ctx.getId() != null ? ctx.getId() : -1L);
                } else {
                    String logoutUrl = "logout";
                    if (url.lastIndexOf(logoutUrl) != -1) {
                        optLog.setCallFunction(logoutUrl);
                    }

                    if (HelperUtils.stringNotEmptyOrNull(ctx.getJwt())) {
                        optLog.setDescription(ctx.getUsername() + " " + optLog.getDescription());
                        optLog.setUserId(ctx.getId() != null ? ctx.getId() : -1L);
                    }
                }
            }

            //操作IP
            optLog.setRequestIp(HelperUtils.getHttpRequestSrcIp(request));
            //返回值信息
            optLog.setResult(HelperUtils.truncateString(HelperUtils.getJson(result), MAX_DB_STRING));
            // 请求Headers
            optLog.setRequestHeaders(HelperUtils.truncateString(HelperUtils.getHttpRequestHeaders(request), MAX_DB_STRING));
            // 响应延时
            optLog.setExpendTime(System.currentTimeMillis() - ctx.getRequestTime());
            // 请求信息
            String reqJson = ctx.getRequestBody();
            optLog.setRequest(HelperUtils.truncateString(reqJson, MAX_DB_STRING));

            if (HelperUtils.stringNotEmptyOrNull(reqJson)) {
                // 传输延时
                Long reqTime = Optional.ofNullable((Long) JsonPath.read(reqJson, "$.timeStamp")).orElse(0L);
                optLog.setTransmitDelay(ctx.getRequestTime() - reqTime);
            }

            //保存日志
            operationLogMapper.insert(optLog, true);
            log.trace("{}", HelperUtils.getJson(optLog));
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    @Override
    public Page<OperationLog> getSystemOperationSummary(Long pageNumber,
                                                               Long pageSize,
                                                               Long totalSize,
                                                               List<String> userName) {
        QueryWrapper wrapper;

        ApiContext ctx = ApiContextUtils.get();

        String username = ctx.getUsername();
        // 当前已经有合法的认证用户
        if (HelperUtils.stringNotEmptyOrNull(username)) {
            if (userName != null && !userName.isEmpty()) {
                List<Long> idArray = userName.stream()
                                             .filter(k -> userDbService.getAccountUserByName(k) != null)
                                             .map(k -> userDbService.getAccountUserByName(k).getId())
                                             .toList();

                wrapper = new QueryWrapper().select(OPERATION_LOG.ALL_COLUMNS)
                                            .from(OPERATION_LOG.as("o"), USER.as("u1"))
                                            .where(OPERATION_LOG.USER_ID.eq(USER.ID)
                                                                        .and(USER.ID.in(idArray))
                                                                        .and(USER.ROLE_ID.ge(
                                                                                select(distinct(USER.ROLE_ID)).from(
                                                                                                                      USER.as("u"))
                                                                                                              .where(USER.ID.eq(
                                                                                                                      ctx.getId())))));
            } else {
                wrapper = new QueryWrapper().select(OPERATION_LOG.ALL_COLUMNS)
                                            .from(OPERATION_LOG.as("o"), USER.as("u1"))
                                            .where(OPERATION_LOG.USER_ID.eq(USER.ID)
                                                                        .and(USER.ROLE_ID.ge(
                                                                                select(distinct(USER.ROLE_ID)).from(
                                                                                                                      USER.as("u"))
                                                                                                              .where(USER.ID.eq(
                                                                                                                      ctx.getId())))));
            }

            return operationLogMapper.paginate(pageNumber, pageSize, totalSize, wrapper);
        } else {
            log.error("Current User is not authentication.");
            throw new CommonErrorCodeException(ErrorCode.ERR_LOGOUT);
        }
    }

    @Override
    public List<OperationLog> getSystemOperationDetails(List<Long> operationId) {
        QueryWrapper wrapper = QueryWrapper.create()
                                           .from(OPERATION_LOG)
                                           .select()
                                           .where(OPERATION_LOG.ID.in(operationId));
        return operationLogMapper.selectListByQuery(wrapper);
    }

    @Override
    public void systemOperationLog(String call,
                                   HttpServletRequest request,
                                   Object result,
                                   String module,
                                   String optType,
                                   String desc,
                                   ErrorCode errorCode) {
        try {
            ApiContext   ctx    = ApiContextUtils.get();
            OperationLog optLog = new OperationLog();

            if (request.getRequestURI().startsWith(CommonConfigure.PROJECT_PREFIX_URL + ERROR_URL)) {
                return;
            }

            optLog.setOperationStatus(errorCode.getDescription());
            optLog.setCallFunction(call);
            //获取注解操作信息
            optLog.setModule(module);
            optLog.setOperationType(optType);
            optLog.setDescription(desc);
            //操作时间
            optLog.setOperationTime(new Timestamp(System.currentTimeMillis()));
            // 请求方法
            optLog.setHttpMethod(request.getMethod());
            // 请求URL
            String prefixUrl = Optional.ofNullable(serverProperties.getServlet().getContextPath()).orElse("");
            String url       = request.getRequestURI().replaceFirst("^" + prefixUrl, "");
            optLog.setHttpPath(url);

            //操作用户
            optLog.setUserId(ctx.getId());

            //操作IP
            optLog.setRequestIp(HelperUtils.getHttpRequestSrcIp(request));
            //返回值信息
            optLog.setResult(HelperUtils.truncateString(HelperUtils.getJson(result), MAX_DB_STRING));
            // 请求Headers
            optLog.setRequestHeaders(HelperUtils.truncateString(HelperUtils.getHttpRequestHeaders(request), MAX_DB_STRING));
            // 请求信息
            String reqJson = ctx.getRequestBody();
            optLog.setRequest(HelperUtils.truncateString(reqJson, MAX_DB_STRING));

            // 响应延时
            optLog.setExpendTime(System.currentTimeMillis() - ctx.getRequestTime());
            if (HelperUtils.stringNotEmptyOrNull(reqJson)) {
                // 传输延时
                Long reqTime = Optional.ofNullable((Long) JsonPath.read(reqJson, "$.timeStamp")).orElse(0L);
                optLog.setTransmitDelay(ctx.getRequestTime() - reqTime);
            }

            //保存日志
            operationLogMapper.insert(optLog, true);
            log.trace("{}", HelperUtils.getJson(optLog));
        } catch (Exception ignored) {
            // not show exception message
        }
    }
}