package com.cf.cs.integratetest.controller;

import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.service.UserDataBaseService;
import com.cf.cs.integratetest.common.TestBaseAuthentication;
import com.cf.cs.protocol.pojo.dto.IdArrayReq;
import com.cf.cs.protocol.pojo.dto.RegisterResourceReq;
import com.cf.cs.protocol.pojo.dto.UserIdReq;
import com.cf.cs.protocol.pojo.po.ResourceInfo;
import com.cf.cs.protocol.pojo.vo.GetRoleResp;
import com.cf.cs.protocol.pojo.vo.RegisterResourceResp;
import com.cf.cs.protocol.pojo.vo.UserResPermInfoResp;
import jakarta.annotation.Resource;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMethod;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@ExtendWith({SpringExtension.class})
@Transactional
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
@SuppressWarnings("java:S5786")
public class PermissionManagerApiTest extends TestBaseAuthentication {
    private final List<String> addResUid = new ArrayList<>();

    @Resource
    private UserDataBaseService userDbService;

    @Test
    @DisplayName("获取当前用户资源信息")
    void testGetCurrentUserResourcePermission() throws Exception {
        UserResPermInfoResp resp = (UserResPermInfoResp) performanceRestful(RequestMethod.GET, null, "/api/permission/resourcePermission",
                                                                            new Class[] {UserResPermInfoResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResPermission());
        AssertValidString(resp.getUserId());
        AssertValidString(resp.getUsername());

        Assertions.assertFalse(resp.getResPermission().isEmpty());

        resp.getResPermission().forEach(k -> {
            Assertions.assertNotNull(k.getId());
            AssertValidString(k.getHttpMethod());
            AssertValidString(k.getPath());
            AssertValidString(k.getName());
            Assertions.assertNotNull(k.getResTypeId());
            Assertions.assertNotNull(k.getEnable());
        });
    }

    @Test
    @DisplayName("获取用户资源信息")
    void testGetUserResourcePermission() throws Exception {
        User user = userDbService.getAccountUserByName("admin");
        Assertions.assertNotNull(user);
        AssertValidString(user.getUid());
        UserResPermInfoResp resp = (UserResPermInfoResp) performanceRestful(RequestMethod.POST, new UserIdReq(user.getUid()),
                                                                            "/api/permission/resourcePermission",
                                                                            new Class[] {UserResPermInfoResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResPermission());
        AssertValidString(resp.getUserId());
        AssertValidString(resp.getUsername());

        Assertions.assertFalse(resp.getResPermission().isEmpty());

        resp.getResPermission().forEach(k -> {
            Assertions.assertNotNull(k.getId());
            AssertValidString(k.getHttpMethod());
            AssertValidString(k.getPath());
            AssertValidString(k.getName());
            Assertions.assertNotNull(k.getResTypeId());
            Assertions.assertNotNull(k.getEnable());
        });
    }

    @Test
    @DisplayName("不存在的用户获取资源信息")
    void testGetUserResourcePermission_UserServiceGetAccountUserByUidReturnsNull() {
        User user = userDbService.getAccountUserByUid("ff99fd68-14bf-4fd6-9320-bab85839245c");
        Assertions.assertNull(user);
    }

    @Test
    @DisplayName("获取用户资源信息, 当前用户无资源")
    void testGetUserResourcePermission_UserServiceGetUserResourcePermReturnsNoItems() throws Exception {
        User user = userDbService.getAccountUserByName("guest");
        Assertions.assertNotNull(user);
        AssertValidString(user.getUid());
        UserResPermInfoResp resp = (UserResPermInfoResp) performanceRestful(RequestMethod.POST, new UserIdReq(user.getUid()),
                                                                            "/api/permission/resourcePermission",
                                                                            new Class[] {UserResPermInfoResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResPermission());
        AssertValidString(resp.getUserId());
        AssertValidString(resp.getUsername());

        Assertions.assertFalse(resp.getResPermission().isEmpty());

        resp.getResPermission().forEach(k -> {
            Assertions.assertNotNull(k.getId());
            AssertValidString(k.getHttpMethod());
            AssertValidString(k.getPath());
            AssertValidString(k.getName());
            Assertions.assertNotNull(k.getResTypeId());
            Assertions.assertNotNull(k.getEnable());
            Assertions.assertFalse(k.getEnable());
        });
    }

    @Test
    @DisplayName("获取当前所有用户组")
    void testGetAllRoles() throws Exception {
        GetRoleResp resp = (GetRoleResp) performanceRestful(RequestMethod.GET, null, "/api/permission/allRoles",
                                                            new Class[] {GetRoleResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getRoles());
        Assertions.assertFalse(resp.getRoles().isEmpty());
        resp.getRoles().forEach(k -> {
            Assertions.assertNotNull(k.getId());
            AssertValidString(k.getDescription());
            Assertions.assertNotNull(k.getName());
            Assertions.assertNotNull(k.getAvailable());
        });
    }

    @Test
    @DisplayName("删除不存在的资源")
    void testRemoveResourceById_UserServiceReturnsNoItems() throws Exception {
        List<String> rid = new ArrayList<>();

        for (int i = 0; i < 10; i++) {
            rid.add(UUID.randomUUID().toString());
        }

        IdArrayReq req = new IdArrayReq(rid);

        RegisterResourceResp resp = (RegisterResourceResp) performanceRestful(RequestMethod.DELETE, req, "/api/permission/resource",
                                                                              new Class[] {RegisterResourceResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResult());
        Assertions.assertEquals(resp.getResult().size(), rid.size());

        resp.getResult().forEach(k -> {
            Assertions.assertEquals(k.getStatus(), ErrorCode.ERR_NOSUCHITEM.getCode());
            Assertions.assertNotNull(k.getRid());
        });
    }

    @Test
    @DisplayName("注册新资源")
    void testRegisterResource() throws Exception {
        addResUid.clear();
        List<ResourceInfo> resList = new ArrayList<>();
        resList.add(ResourceInfo.builder()
                                .path("/api/custom/show")
                                .name("show")
                                .resTypeId(0)
                                .httpMethod(new ArrayList<>() {{add("POST");}})
                                .build());

        resList.add(ResourceInfo.builder()
                                .path("/api/custom/List")
                                .name("List")
                                .resTypeId(1)
                                .httpMethod(new ArrayList<>() {{add("PUT");}})
                                .build());

        RegisterResourceReq req = RegisterResourceReq.builder().resources(resList).build();

        RegisterResourceResp resp = (RegisterResourceResp) performanceRestful(RequestMethod.PUT, req, "/api/permission/resource",
                                                                              new Class[] {RegisterResourceResp.class});

        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResult());
        Assertions.assertEquals(resp.getResult().size(), resList.size());

        resp.getResult().forEach(k -> {
            AssertValidCommonResp(k);
            AssertValidString(k.getRid());
            AssertValidString(k.getPath());
            AssertValidString(k.getHttpMethod());
            Assertions.assertNotNull(k.getResTypeId());
            addResUid.add(k.getRid());
        });
    }

    @Test
    @DisplayName("删除资源")
    void testRemoveResourceById() throws Exception {
        testRegisterResource();
        IdArrayReq req = new IdArrayReq(addResUid);

        RegisterResourceResp resp = (RegisterResourceResp) performanceRestful(RequestMethod.DELETE, req, "/api/permission/resource",
                                                                              new Class[] {RegisterResourceResp.class});
        AssertValidCommonResp(resp);
        Assertions.assertNotNull(resp.getResult());
        Assertions.assertEquals(resp.getResult().size(), addResUid.size());

        resp.getResult().forEach(k -> {
            AssertValidCommonResp(k);
            AssertValidString(k.getRid());
            Assertions.assertTrue(addResUid.contains(k.getRid()));
        });
    }
}
