package com.cf.cs.protocol.misc;

import com.cf.cs.base.config.ObjectMapperProvider;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.ServletInputStream;

import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Stack;

/**
 * The type Protocol json utils.
 *
 * @author xajhuang @163.com
 */
public class ProtocolJsonUtils {
    private ProtocolJsonUtils() {
        throw new AssertionError("Instantiating utility class.");
    }

    private static final ObjectMapper OBJ_MAPPER = ObjectMapperProvider.getMapper();

    /**
     * Get json bytes byte [ ].
     *
     * @param <T>     the type parameter
     * @param obj     the obj
     * @param charset the charset
     * @return the byte [ ]
     * @throws JsonProcessingException the json processing exception
     */
    public static <T> byte[] getJsonBytes(T obj, Charset charset) throws JsonProcessingException {
        return OBJ_MAPPER.writeValueAsString(obj).getBytes(charset);
    }

    /**
     * Get json bytes byte [ ].
     *
     * @param <T> the type parameter
     * @param obj the obj
     * @return the byte [ ]
     * @throws JsonProcessingException the json processing exception
     */
    public static <T> byte[] getJsonBytes(T obj) throws JsonProcessingException {
        return OBJ_MAPPER.writeValueAsString(obj).getBytes(StandardCharsets.UTF_8);
    }

    /**
     * Json get object object.
     *
     * @param <T>       the type parameter
     * @param json      the json
     * @param valueType the value type
     * @return the object
     * @throws JsonProcessingException the json processing exception
     */
    public static <T> Object jsonGetObject(String json, Class<T> valueType) throws JsonProcessingException {
        return OBJ_MAPPER.readValue(json, valueType);
    }

    /**
     * Json get object t.
     *
     * @param <T>          the type parameter
     * @param is           the is
     * @param valueTypeRef the value type ref
     * @return the t
     * @throws IOException the io exception
     */
    public static <T> T jsonGetObject(ServletInputStream is, TypeReference<T> valueTypeRef) throws IOException {
        return OBJ_MAPPER.readValue(HelperUtils.inputStream2String(is), valueTypeRef);
    }

    /**
     * Json get object t.
     *
     * @param <T>          the type parameter
     * @param json         the json
     * @param valueTypeRef the value type ref
     * @return the t
     * @throws IOException the io exception
     */
    public static <T> T jsonGetObject(String json, TypeReference<T> valueTypeRef) throws IOException {
        return OBJ_MAPPER.readValue(json, valueTypeRef);
    }

    private static <T> Type createRespType(Class<T> c) {
        return new ParameterizedType() {
            @Override
            public Type[] getActualTypeArguments() {
                // 返回实际类型参数，例如List<String>的String
                return new Type[] {c};
            }

            @Override
            public Type getRawType() {
                // 返回原始类型，例如List
                return ProtocolResp.class;
            }

            @Override
            public Type getOwnerType() {
                // 返回类型所在的类，对于顶级类是null
                return null;
            }
        };
    }

    /**
     * Json get protocol resp protocol resp.
     *
     * @param <T>        the type parameter
     * @param jsonString the json string
     * @param subClass   the sub class
     * @return the protocol resp
     * @throws JsonProcessingException the json processing exception
     */
    @SuppressWarnings("java:S1452")
    public static <T> ProtocolResp<? extends BaseRespStatus> jsonGetProtocolResp(String jsonString,
                                                                                 Class<T> subClass) throws JsonProcessingException {
        return OBJ_MAPPER.readValue(jsonString, new TypeReference<>() {
            @Override
            public Type getType() {
                return createRespType(subClass);
            }
        });
    }

    @SuppressWarnings("java:S1149")
    private static Type createRespType(Class<?>[] c) {
        if (c.length == 1) {
            return createRespType(c[0]);
        }

        Stack<Class<?>> st = new Stack<>();

        Arrays.stream(c).forEach(st::push);

        Class<?> fc = st.pop();
        Class<?> sc = st.pop();
        JsonParameterizedType type = JsonParameterizedType.make(sc,
                                                                new Type[] {fc},
                                                                sc.getDeclaringClass());

        while (!st.isEmpty()) {
            Class<?> mc = st.pop();
            type = JsonParameterizedType.make(mc,
                                              new Type[] {type},
                                              mc.getDeclaringClass());
        }

        type = JsonParameterizedType.make(ProtocolResp.class,
                                          new Type[] {type},
                                          ProtocolResp.class.getDeclaringClass());

        return type;
    }

    /**
     * Json get protocol resp protocol resp.
     *
     * @param jsonString the json string
     * @param subClass   the sub class
     * @return the protocol resp
     * @throws JsonProcessingException the json processing exception
     */
    @SuppressWarnings("java:S1452")
    public static ProtocolResp<? extends BaseRespStatus> jsonGetProtocolResp(String jsonString,
                                                                             Class<?>[] subClass) throws JsonProcessingException {
        return OBJ_MAPPER.readValue(jsonString, new TypeReference<>() {
            @Override
            public Type getType() {
                return createRespType(subClass);
            }
        });
    }
}
