package com.cf.cs.protocol.service.impl;

import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.common.ProtoCryptoType;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.crypto.arithmetic.CryptoHelper;
import com.cf.cs.protocol.config.ProtocolConfigure;
import com.cf.cs.protocol.crypto.DecryptRequestProtocol;
import com.cf.cs.protocol.exception.SecurityProtocolException;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.service.ProtocolSecurityService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.http.HttpInputMessage;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.Method;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * The type Protocol security service.
 *
 * @author xajhuang @163.com
 */
@Service
@Slf4j
public class ProtocolSecurityServiceImpl implements ProtocolSecurityService {
    @Resource
    private ObjectMapper objectMapper;

    @Resource
    private ProtocolConfigure protocolConfigure;

    @Override
    public String decryptProtocol(String ciphertext) throws JsonProcessingException {
        JsonNode objRoot = objectMapper.readTree(ciphertext);

        int cryptoType = objRoot.path("cryptoType").asInt();

        // 协议未加密
        if (cryptoType == ProtoCryptoType.CRYPTO_NONE.getValue()) {
            return ciphertext;
        }

        if (cryptoType > ProtoCryptoType.CRYPTO_AES256.getValue() || cryptoType < ProtoCryptoType.CRYPTO_NONE.getValue()) {
            throw new SecurityProtocolException(ErrorCode.ERR_PARAMS, "cryptoType 字段取值为 [0, 4]");
        }

        // 反序列化为对象处理
        ProtocolReq<String> proReq = objectMapper.readValue(ciphertext, new TypeReference<>() {
        });

        // 首先对加密内容进行base64解码
        byte[] base64Decode = CryptoHelper.base64Decryption(proReq.getMsgContent());

        // 解密后的内容
        byte[] decryptContent;

        // 加密类型为base64直接返回处理结果
        if (Objects.equals(proReq.getCryptoType(), ProtoCryptoType.CRYPTO_BASE64.getValue())) {
            decryptContent = base64Decode;
        } else if (Objects.equals(proReq.getCryptoType(), ProtoCryptoType.CRYPTO_AES128.getValue())) {
            try {
                decryptContent = CryptoHelper.aes128Decryption(base64Decode, protocolConfigure.getCryptoKey());
            } catch (Exception e) {
                log.error("AES128 decode message error: {}", base64Decode);
                throw new SecurityProtocolException(ErrorCode.ERR_DECRYPT_AES128);
            }
        } else if (Objects.equals(proReq.getCryptoType(), ProtoCryptoType.CRYPTO_AES256.getValue())) {
            try {
                decryptContent = CryptoHelper.aes256Decryption(base64Decode, protocolConfigure.getCryptoKey());
            } catch (Exception e) {
                log.error("AES256 decode message error: {}", base64Decode);
                throw new SecurityProtocolException(ErrorCode.ERR_DECRYPT_AES256);
            }
        } else if (Objects.equals(proReq.getCryptoType(), ProtoCryptoType.CRYPTO_DES.getValue())) {
            try {
                decryptContent = CryptoHelper.desDecryption(base64Decode, protocolConfigure.getCryptoKey());
            } catch (Exception e) {
                log.error("DES decode message error: {}", base64Decode);
                throw new SecurityProtocolException(ErrorCode.ERR_DECRYPT_3DES);
            }
        } else {
            log.error("Unknown protocol security type: {}, {}", proReq.getCryptoType(), ciphertext);
            throw new SecurityProtocolException(ErrorCode.ERR_DECRYPT_UNKNOWN);
        }

        // 字节数组转换为字符串
        String decodeMsg = new String(decryptContent, StandardCharsets.UTF_8);

        return ciphertext.replace("\"" + proReq.getMsgContent() + "\"", decodeMsg);
    }

    @Override
    public DecryptRequestProtocol decryptProtocol(HttpInputMessage httpInputMessage) throws IOException {
        // 提取协议中的JSON字符串
        String reqMessage = IOUtils.toString(httpInputMessage.getBody(), StandardCharsets.UTF_8);

        return new DecryptRequestProtocol(httpInputMessage, decryptProtocol(reqMessage));
    }

    @Override
    public String encryptProtocolString(String plainText, int cryptoType) {
        String cipherText;
        if (cryptoType == ProtoCryptoType.CRYPTO_BASE64.getValue()) {
            cipherText = CryptoHelper.base64Encryption(plainText.getBytes(StandardCharsets.UTF_8));
        } else if (cryptoType == ProtoCryptoType.CRYPTO_AES128.getValue()) {
            try {
                byte[] encode = CryptoHelper.aes128Encryption(plainText.getBytes(StandardCharsets.UTF_8), protocolConfigure.getCryptoKey());
                cipherText = CryptoHelper.base64Encryption(encode);
            } catch (Exception e) {
                log.error("AES128 encode message error({}): {}", e.getMessage(), plainText);
                throw new SecurityProtocolException(ErrorCode.ERR_ENCRYPT_AES128, e.getMessage());
            }
        } else if (cryptoType == ProtoCryptoType.CRYPTO_AES256.getValue()) {
            try {
                byte[] encode = CryptoHelper.aes256Encryption(plainText.getBytes(StandardCharsets.UTF_8), protocolConfigure.getCryptoKey());
                cipherText = CryptoHelper.base64Encryption(encode);
            } catch (Exception e) {
                log.error("AES256 encode message error({}): {}", e.getMessage(), plainText);
                throw new SecurityProtocolException(ErrorCode.ERR_ENCRYPT_AES256, e.getMessage());
            }
        } else if (cryptoType == ProtoCryptoType.CRYPTO_DES.getValue()) {
            try {
                byte[] encode = CryptoHelper.desEncryption(plainText.getBytes(StandardCharsets.UTF_8), protocolConfigure.getCryptoKey());
                cipherText = CryptoHelper.base64Encryption(encode);
            } catch (Exception e) {
                log.error("DES encode message error({}): {}", e.getMessage(), plainText);
                throw new SecurityProtocolException(ErrorCode.ERR_ENCRYPT_3DES, e.getMessage());
            }
        } else {
            log.error("Unknown protocol security type: {}, {}", cryptoType, plainText);
            throw new SecurityProtocolException(ErrorCode.ERR_ENCRYPT_UNKNOWN);
        }

        return cipherText;
    }


    @Override
    public ProtocolResp<String> encryptProtocol(Object orgProtocol, int cryptoType) {
        ProtocolResp<String> cryptoObject = new ProtocolResp<>();
        cryptoObject.setVer(ConstValue.Protocol.VERSION);
        cryptoObject.setCryptoType(ProtocolConfigure.SECURITY_PROTOCOL_TYPE);
        cryptoObject.setTimeStamp(System.currentTimeMillis());

        String msgContentJsonString;

        try {
            Method getMsgMethod = orgProtocol.getClass().getMethod("getMsgContent");
            Method getCode      = orgProtocol.getClass().getMethod("getCode");
            msgContentJsonString = HelperUtils.getJson(getMsgMethod.invoke(orgProtocol));
            cryptoObject.setCode((Integer) getCode.invoke(orgProtocol));
        } catch (Exception e) {
            log.error("Json encode message error: {}", orgProtocol);
            throw new SecurityProtocolException(ErrorCode.ERR_SYSTEMEXCEPTION);
        }

        cryptoObject.setMsgContent(encryptProtocolString(msgContentJsonString, cryptoType));

        return cryptoObject;
    }
}
