package com.cf.cs.restful.controller;


import com.cf.cs.authentication.pojo.po.AuthAccountUser;
import com.cf.cs.authentication.service.AuthUsersService;
import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.crypto.annotation.DecryptionProtocol;
import com.cf.cs.crypto.annotation.EncryptionProtocol;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.service.UserDataBaseService;
import com.cf.cs.protocol.pojo.dto.IdArrayReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.dto.RegisterResourceReq;
import com.cf.cs.protocol.pojo.dto.UserIdReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.vo.GetRoleResp;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.pojo.vo.RegisterResourceResp;
import com.cf.cs.protocol.pojo.vo.UserResPermInfoResp;
import com.cf.cs.protocol.validation.group.ValidGroups;
import com.cf.cs.restful.service.UserPermissionService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * The type Permission manager api.
 *
 * @author xajhuang @163.com
 */
@RestController
@Slf4j
@Tag(name = "权限管理接口")
@RequestMapping(value = "/api/permission")
@SuppressWarnings("java:S1452")
public class PermissionManagerApi {
    @Resource
    private UserPermissionService userPermissionService;

    @Resource
    private UserDataBaseService userDbService;

    @Resource
    private AuthUsersService authUsersService;

    /**
     * Gets current user resource permission.
     *
     * @return the current user resource permission
     */
    @GetMapping("resourcePermission")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "权限管理模块", OperationType = "读取", OperationDesc = "获取当前用户资源权限")
    public ProtocolResp<? extends BaseRespStatus> getCurrentUserResourcePermission() {
        AuthAccountUser au = authUsersService.getAuthAccountByUserName(ApiContextUtils.get().getUsername());

        if (au == null) {
            return ProtocolResp.result(ErrorCode.ERR_TOKENNOTFOUND);
        }

        return ProtocolResp.result(UserResPermInfoResp.builder()
                                                      .userId(au.getUid())
                                                      .username(au.getUsername())
                                                      .resPermission(userDbService.getCurrentUserResourcePerm())
                                                      .build());
    }

    /**
     * Gets user resource permission.
     *
     * @param mr the mr
     * @return the user resource permission
     */
    @PostMapping("resourcePermission")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "权限管理模块", OperationType = "读取", OperationDesc = "获取用户资源权限")
    public ProtocolResp<? extends BaseRespStatus> getUserResourcePermission(@RequestBody ProtocolReq<UserIdReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.OperationLogReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            User user = userDbService.getAccountUserByUid(mr.getMsgContent().getUid());

            if (user == null) {
                return ProtocolResp.result(ErrorCode.ERR_USERNOTFOUND);
            }

            return ProtocolResp.result(UserResPermInfoResp.builder()
                                                          .userId(user.getUid())
                                                          .username(user.getUsername())
                                                          .resPermission(userDbService.getUserResourcePerm(user.getId()))
                                                          .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets all roles.
     *
     * @return the all roles
     */
    @GetMapping("/allRoles")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "权限管理模块", OperationType = "读取", OperationDesc = "获取当前所有用户组")
    public ProtocolResp<? extends BaseRespStatus> getAllRoles() {
        return ProtocolResp.result(GetRoleResp.builder().roles(userPermissionService.getAllUserGroup()).build());
    }

    /**
     * Register resource protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @PutMapping("/resource")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "权限管理模块", OperationType = "创建", OperationDesc = "注册新的资源")
    public ProtocolResp<? extends BaseRespStatus> registerResource(@RequestBody ProtocolReq<RegisterResourceReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.ResourceReqValid.class,
                                                     ValidGroups.ResourceInfoValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            RegisterResourceReq res = mr.getMsgContent();
            return ProtocolResp.result(ErrorCode.ERR_OK,
                                       RegisterResourceResp.builder()
                                                           .result(userPermissionService.registerResource(res.getResources()))
                                                           .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Remove resource by id protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @DeleteMapping("/resource")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "权限管理模块", OperationType = "删除", OperationDesc = "删除资源")
    public ProtocolResp<? extends BaseRespStatus> removeResourceById(@RequestBody ProtocolReq<IdArrayReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.UserIdReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            List<String> ids = mr.getMsgContent().getIds();
            return ProtocolResp.result(ErrorCode.ERR_OK,
                                       RegisterResourceResp.builder().result(userPermissionService.removeResourceByUid(ids)).build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }
}
