package com.cf.cs.restful.controller;

import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.crypto.annotation.DecryptionProtocol;
import com.cf.cs.crypto.annotation.EncryptionProtocol;
import com.cf.cs.database.pojo.po.UserDictionary;
import com.cf.cs.protocol.pojo.dto.AddDictContentReq;
import com.cf.cs.protocol.pojo.dto.BasePagedReq;
import com.cf.cs.protocol.pojo.dto.DictContentReq;
import com.cf.cs.protocol.pojo.dto.NewUserDictReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.dto.RemoveDictReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.po.DictContent;
import com.cf.cs.protocol.pojo.vo.BasePageResultResp;
import com.cf.cs.protocol.pojo.vo.DictContentResp;
import com.cf.cs.protocol.pojo.vo.NewUserDictResp;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.pojo.vo.DictTypeResp;
import com.cf.cs.protocol.validation.group.ValidGroups;
import com.cf.cs.restful.service.DictionaryService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;

/**
 * The type System dict controller.
 *
 * @author xajhuang @163.com
 */
@RestController
@Slf4j
@Tag(name = "字典接口")
@RequestMapping(value = "/api/dict")
@SuppressWarnings("java:S1452")
public class SystemDictController {
    @Resource
    private DictionaryService dictionaryService;

    /**
     * Gets enum dictionary.
     *
     * @return the enum dictionary
     */
    @GetMapping("/enum/allDict")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "读取", OperationDesc = "获取系统所有枚举字典类型信息")
    public ProtocolResp<? extends BaseRespStatus> getEnumDictionary() {
        Set<String> ret = dictionaryService.getAllEnumDictionary();

        return ProtocolResp.result(DictTypeResp.builder()
                                               .enumDict(ret)
                                               .build());
    }

    /**
     * Gets enum dictionary content.
     *
     * @param mr the mr
     * @return the enum dictionary content
     */
    @PostMapping("/enum/dictContent")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "读取", OperationDesc = "获取系统枚举字典详细内容")
    public ProtocolResp<? extends BaseRespStatus> getEnumDictionaryContent(@RequestBody ProtocolReq<DictContentReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class,
                                                     ValidGroups.DictReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            DictContentReq req = mr.getMsgContent();

            List<DictContent> ret = dictionaryService.getEnumDictionaryContent(req.getDictTpe());

            return ProtocolResp.result(DictContentResp.builder()
                                                      .sysDict(ret)
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION,
                                       ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Create new dictionary protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @PutMapping("/user/add")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "创建", OperationDesc = "新建用户字典")
    public ProtocolResp<? extends BaseRespStatus> createNewDictionary(@RequestBody ProtocolReq<NewUserDictReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.DictReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            NewUserDictReq req = mr.getMsgContent();
            dictionaryService.addNewUserDictionary(req.getDictName(), req.getDictAlias(), req.getDictRemark());
            return ProtocolResp.result(NewUserDictResp.builder()
                                                      .dictName(req.getDictName())
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Remove dictionary protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @DeleteMapping("/user/delete")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "创建", OperationDesc = "新建用户字典")
    public ProtocolResp<? extends BaseRespStatus> removeDictionary(@RequestBody ProtocolReq<RemoveDictReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.DictReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            RemoveDictReq req = mr.getMsgContent();
            dictionaryService.removeUserDictionary(req.getDictName());
            return ProtocolResp.result(NewUserDictResp.builder()
                                                      .dictName(req.getDictName())
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Upgrade dictionary protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @PostMapping("/user/upgrade")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "修改", OperationDesc = "修改用户字典内容")
    public ProtocolResp<? extends BaseRespStatus> upgradeDictionary(@RequestBody ProtocolReq<NewUserDictReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.DictReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            NewUserDictReq req = mr.getMsgContent();
            dictionaryService.upgradeUserDictionary(req.getDictName(), req.getDictAlias(), req.getDictRemark());
            return ProtocolResp.result(NewUserDictResp.builder()
                                                      .dictName(req.getDictName())
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets user dictionary.
     *
     * @param mr the mr
     * @return the user dictionary
     */
    @PostMapping("/user/allDict")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "读取", OperationDesc = "获取所有用户字典")
    public ProtocolResp<? extends BaseRespStatus> getUserDictionary(@RequestBody ProtocolReq<BasePagedReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.BasePagedReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            BasePagedReq req = mr.getMsgContent();
            PageResults<UserDictionary> ret = dictionaryService.getUserDictionary(req.getPageNumber(),
                                                                                  req.getPageSize(),
                                                                                  req.getTotalSize());
            return ProtocolResp.result(BasePageResultResp.<UserDictionary>builder()
                                                         .items(ret)
                                                         .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets user dictionary content.
     *
     * @param mr the mr
     * @return the user dictionary content
     */
    @PostMapping("/user/getDictContent")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "读取", OperationDesc = "获取用户字典内容项")
    public ProtocolResp<? extends BaseRespStatus> getUserDictionaryContent(@RequestBody ProtocolReq<DictContentReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.DictReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            DictContentReq    req = mr.getMsgContent();
            List<DictContent> ret = dictionaryService.getUserDictionaryContent(req.getDictTpe());

            return ProtocolResp.result(DictContentResp.builder()
                                                      .sysDict(ret)
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Create new dictionary content protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @PutMapping("/user/addDictContent")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "字典模块", OperationType = "创建", OperationDesc = "新增用户字典内容项")
    public ProtocolResp<? extends BaseRespStatus> createNewDictionaryContent(@RequestBody ProtocolReq<AddDictContentReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.DictReqValid.class,
                                                     ValidGroups.DictReqContentValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            AddDictContentReq req = mr.getMsgContent();
            //dictionaryService.addNewUserDictionary(req.getDictName(), req.getDictAlias(), req.getDictRemark());
            return ProtocolResp.result(NewUserDictResp.builder()
                                                      .dictName(req.getDictName())
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }
}
