package com.cf.cs.restful.controller;


import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.crypto.annotation.DecryptionProtocol;
import com.cf.cs.crypto.annotation.EncryptionProtocol;
import com.cf.cs.database.pojo.po.UserInfo;
import com.cf.cs.database.service.UserDataBaseService;
import com.cf.cs.protocol.pojo.dto.BasePagedReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.dto.RegisterUserReq;
import com.cf.cs.protocol.pojo.dto.UserIdReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.vo.BasePageResultResp;
import com.cf.cs.protocol.pojo.vo.GetUserInfoResp;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.validation.group.ValidGroups;
import com.cf.cs.restful.service.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * The type User manager api.
 *
 * @author xajhuang @163.com
 */
@RestController
@Slf4j
@Tag(name = "用户管理接口")
@RequestMapping(value = "/api/user")
@SuppressWarnings("java:S1452")
public class UserManagerApi {
    @Resource
    private UserDataBaseService userDbService;

    @Resource
    private UserService userService;

    /**
     * Gets user info by id.
     *
     * @param mr the mr
     * @return the user info by id
     */
    @PostMapping("/userInfo")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "用户管理模块", OperationType = "读取", OperationDesc = "获取用户信息")
    public ProtocolResp<? extends BaseRespStatus> getUserInfoById(@RequestBody ProtocolReq<UserIdReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.OperationLogReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            UserInfo ui = userDbService.getUserInfoByUid(mr.getMsgContent().getUid());
            return ProtocolResp.result(GetUserInfoResp.builder()
                                                      .userInfo(ui)
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets all user info paged.
     *
     * @param mr the mr
     * @return the all user info paged
     */
    @PostMapping("/userList")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "用户管理模块", OperationType = "读取", OperationDesc = "获取用户列表")
    public ProtocolResp<? extends BaseRespStatus> getAllUserInfoPaged(@RequestBody ProtocolReq<BasePagedReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.BasePagedReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            BasePagedReq          req = mr.getMsgContent();
            PageResults<UserInfo> ui  = userDbService.getAllUserInfo(req.getPageNumber(), req.getPageSize(), req.getTotalSize());

            return ProtocolResp.result(BasePageResultResp.<UserInfo>builder()
                                                         .items(ui)
                                                         .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets current user info.
     *
     * @return the current user info
     */
    @GetMapping("/userInfo")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "用户管理模块", OperationType = "读取", OperationDesc = "获取当前用户信息")
    public ProtocolResp<GetUserInfoResp> getCurrentUserInfo() {
        return ProtocolResp.result(GetUserInfoResp.builder()
                                                  .userInfo(userDbService.getCurrentUserInfo())
                                                  .build());
    }

    /**
     * Create new user protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @PutMapping("/register")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "用户管理模块", OperationType = "创建", OperationDesc = "创建新用户")
    public ProtocolResp<? extends BaseRespStatus> createNewUser(@RequestBody ProtocolReq<RegisterUserReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.UserReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            RegisterUserReq ui = mr.getMsgContent();
            userService.registerUser(ui.getUsername(), ui.getPassword(), ui.getRoleId());

            return ProtocolResp.result(GetUserInfoResp.builder()
                                                      .userInfo(userDbService.getUserInfoByUsername(ui.getUsername()))
                                                      .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Remove user protocol resp.
     *
     * @param mr the mr
     * @return the protocol resp
     */
    @DeleteMapping("/remove")
    @EncryptionProtocol
    @DecryptionProtocol
    @OperationLogAnnotation(OperationModule = "用户管理模块", OperationType = "删除", OperationDesc = "删除用户")
    public ProtocolResp<BaseRespStatus> removeUser(@RequestBody ProtocolReq<UserIdReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class, ValidGroups.UserIdReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            UserIdReq ui = mr.getMsgContent();
            userService.removeUser(ui.getUid());
            return ProtocolResp.result(ErrorCode.ERR_OK);
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION, ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }
}
