package com.cf.cs.restful.service.impl;

import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.exception.CommonErrorCodeException;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.database.mapper.DictDataMapper;
import com.cf.cs.database.mapper.DictTypeMapper;
import com.cf.cs.database.pojo.entity.DictType;
import com.cf.cs.database.pojo.entity.DictData;
import com.cf.cs.database.pojo.po.UserDictionary;
import com.cf.cs.protocol.pojo.po.DictContent;
import com.cf.cs.protocol.pojo.po.DictDetails;
import com.cf.cs.restful.service.DictionaryService;
import com.houkunlin.system.dict.starter.DictUtil;
import com.houkunlin.system.dict.starter.SystemDictStarter;
import com.houkunlin.system.dict.starter.bean.DictTypeVo;
import com.houkunlin.system.dict.starter.notice.RefreshDictTypeEvent;
import com.houkunlin.system.dict.starter.notice.RefreshDictValueEvent;
import com.houkunlin.system.dict.starter.provider.SystemDictProvider;
import com.houkunlin.system.dict.starter.store.DictStore;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.Resource;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.cf.cs.database.pojo.entity.table.DictTypeTableDef.DICT_TYPE;


/**
 * The type Dictionary service.
 *
 * @author xajhuang @163.com
 */
@Service
public class DictionaryServiceImpl implements DictionaryService {
    @Resource
    private DictTypeMapper dictTypeMapper;

    @Resource
    private DictDataMapper dictDataMapper;

    @Resource
    private ApplicationEventPublisher publisher;

    @Override
    public Set<String> getAllEnumDictionary() {
        return SystemDictStarter.getBean(DictStore.class).systemDictTypeKeys();
    }

    @Override
    public List<DictContent> getEnumDictionaryContent(List<String> dictType) {
        List<String>      allSysDict  = new ArrayList<>();
        List<DictContent> sysDictList = new ArrayList<>();

        SystemDictStarter.getBean(SystemDictProvider.class).dictTypeIterator().forEachRemaining(k -> {
            if (dictType.contains(k.getType())) {
                List<DictDetails> children = new ArrayList<>();
                k.getChildren().forEach(m -> children.add(DictDetails.builder()
                                                                     .title(m.getTitle())
                                                                     .value((Integer) m.getValue())
                                                                     .disabled(m.isDisabled())
                                                                     .build()));
                sysDictList.add(DictContent.builder()
                                           .dictName(k.getType())
                                           .children(children)
                                           .build());
            }
            allSysDict.add(k.getType());
        });

        dictType.removeAll(allSysDict);

        dictType.forEach(k -> sysDictList.add(DictContent.builder()
                                                         .dictName(k)
                                                         .build()));

        return sysDictList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addNewUserDictionary(String type, String name, String remark) {
        DictType dt = new DictType();
        dt.setRemark(remark);
        dt.setDictName(name);
        dt.setDictTypeName(type);
        dt.setCreateBy(ApiContextUtils.get().getUsername());

        try {
            if (dictTypeMapper.insert(dt) != 1) {
                throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
            }

            // 更新字典
            DictTypeVo.DictTypeBuilder builder = DictTypeVo.newBuilder(type, name, remark);
            publisher.publishEvent(new RefreshDictTypeEvent(builder.build()));
        } catch (DataAccessException e) {
            throw new CommonErrorCodeException(ErrorCode.ERR_ITEMEXISTS, type + ": " + ErrorCode.ERR_ITEMEXISTS.getDescription());
        }
    }

    @Override
    public void upgradeUserDictionary(String type, String name, String remark) {
        boolean changed = false;

        DictType dt = dictTypeMapper.selectOneByCondition(DICT_TYPE.DICT_TYPE_NAME.eq(type));

        // 找不到对应的字典供修改
        if (dt == null) {
            throw new CommonErrorCodeException(ErrorCode.ERR_NOSUCHITEM,
                                               type + ": " + ErrorCode.ERR_NOSUCHITEM.getDescription());
        }

        if (name != null) {
            dt.setDictName(name);
            changed = true;
        }

        if (remark != null) {
            dt.setRemark(remark);
            changed = true;
        }

        // 确认执行过修改
        if (changed) {
            dt.setUpdateBy(ApiContextUtils.get().getUsername());
            dt.setUpdateTime(new Timestamp(System.currentTimeMillis()));

            if (dictTypeMapper.update(dt) != 1) {
                throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
            } else {
                // 数据库操作成功后更新缓存字典数据
                DictTypeVo dv = DictUtil.getDictType(dt.getDictTypeName());
                dv.setRemark(dt.getRemark());
                dv.setTitle(dt.getDictName());
                publisher.publishEvent(new RefreshDictTypeEvent(dv));
            }
        }
    }

    @Override
    public PageResults<UserDictionary> getUserDictionary(Long pageNumber, Long pageSize, Long totalSize) {
        if (totalSize == 0) {
            totalSize = -1L;
        }

        QueryWrapper wrapper = QueryWrapper.create()
                                           .from(DICT_TYPE)
                                           .select();

        Page<UserDictionary> dict = dictTypeMapper.paginateAs(pageNumber, pageSize, totalSize, wrapper, UserDictionary.class);

        return PageResults.<UserDictionary>builder()
                          .items(dict.getRecords())
                          .pageNumber(dict.getPageNumber())
                          .pageSize(dict.getPageSize())
                          .totalPage(dict.getTotalPage())
                          .totalRow(dict.getTotalRow())
                          .build();
    }

    @Override
    public List<DictContent> getUserDictionaryContent(List<String> dictName) {
        List<DictContent> resp = new ArrayList<>();

        QueryWrapper wrapper = QueryWrapper.create()
                                           .from(DICT_TYPE)
                                           .select()
                                           .where(DICT_TYPE.DICT_TYPE_NAME.in(dictName));

        List<DictType> ret = dictTypeMapper.selectListWithRelationsByQuery(wrapper);

        ret.forEach(k -> {
            List<DictDetails> children = new ArrayList<>();

            k.getDictDataList().forEach(m -> children.add(DictDetails.builder()
                                                                     .id(m.getId())
                                                                     .title(m.getDictLabel())
                                                                     .value(m.getDictValue())
                                                                     .sorted(m.getDictSort())
                                                                     .cssClass(HelperUtils.meagreDbStringValue(m.getCssClass()))
                                                                     .listClass(HelperUtils.meagreDbStringValue(m.getListClass()))
                                                                     .isDefault("Y".equals(m.getIsDefault()))
                                                                     .createBy(m.getCreateBy())
                                                                     .createTime(m.getCreateTime())
                                                                     .upgradeBy(HelperUtils.meagreDbStringValue(m.getUpdateBy()))
                                                                     .upgradeTime(m.getUpdateTime())
                                                                     .disabled(false)
                                                                     .build()));

            resp.add(DictContent.builder()
                                .dictId(k.getId())
                                .dictName(k.getDictTypeName())
                                .dictAlias(k.getDictName())
                                .dictRemark(k.getRemark())
                                .children(children)
                                .build());
        });

        dictName.stream()
                .filter(k -> ret.stream().noneMatch(v -> v.getDictTypeName().equals(k)))
                .forEach(m -> {
                    DictContent dt = DictContent.builder()
                                                .dictName(m)
                                                .build();
                    dt.setStatus(ErrorCode.ERR_NOSUCHITEM.getCode());
                    dt.setMessage(new String[] {ErrorCode.ERR_NOSUCHITEM.getDescription()});
                    resp.add(dt);
                });

        return resp;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeUserDictionary(String dictName) {
        DictType dt = dictTypeMapper.selectOneWithRelationsByCondition(DICT_TYPE.DICT_TYPE_NAME.eq(dictName));

        if (dt == null) {
            throw new CommonErrorCodeException(ErrorCode.ERR_NOSUCHITEM,
                                               dictName + ": " + ErrorCode.ERR_NOSUCHITEM.getDescription());
        }

        List<Long> contentId = dt.getDictDataList().stream().map(DictData::getId).toList();

        // 首先删除字典内容
        if (dictDataMapper.deleteBatchByIds(contentId) != contentId.size()) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
        }

        // 删除用户字典
        if (dictTypeMapper.deleteById(dt.getId()) != 1) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE, dictName + ": " + ErrorCode.ERR_DATABASE.getDescription());
        }

        // 更新字典缓存
        DictTypeVo dv = DictUtil.getDictType(dictName);
        dv.getChildren().forEach(v -> v.setDisabled(true));
        publisher.publishEvent(new RefreshDictValueEvent(dv.getChildren(), true));
    }
}
