package com.cf.cs.restful.service.impl;

import com.cf.cs.base.common.CommonStatus;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.exception.CommonErrorCodeException;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.database.mapper.RoleMapper;
import com.cf.cs.database.mapper.UserCredentialMapper;
import com.cf.cs.database.mapper.UserMapper;
import com.cf.cs.database.pojo.entity.Role;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.pojo.entity.UserCredential;
import com.cf.cs.database.service.UserDataBaseService;
import com.cf.cs.restful.service.UserService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import static com.cf.cs.database.pojo.entity.table.UserTableDef.USER;

/**
 * The type User service.
 *
 * @author xajhuang @163.com
 */
@Service
@Slf4j
public class UserServiceImpl implements UserService {
    @Resource
    private UserDataBaseService userDbService;

    @Resource
    private RoleMapper roleMapper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private UserCredentialMapper userCredentialMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void registerUser(String username, String password, Long roleId) {
        Role role = roleMapper.selectOneById(roleId);

        if (role == null) {
            log.error("User Group{} is not exists.", roleId);
            throw new CommonErrorCodeException(ErrorCode.ERR_USER_ROLE_NOTEXISTS);
        }

        User user = userDbService.getAccountUserByName(username);

        if (user != null) {
            log.error("User {} is exists.", username);
            throw new CommonErrorCodeException(ErrorCode.ERR_USEREXIST);
        }

        String loginUsername = ApiContextUtils.get().getUsername();

        if (!HelperUtils.stringNotEmptyOrNull(loginUsername)) {
            log.error("Current login user not found");
            throw new CommonErrorCodeException(ErrorCode.ERR_TOKENNOTFOUND);
        }

        User logUser = userDbService.getAccountUserByName(loginUsername);

        if (logUser == null) {
            log.error("Login User({}) is not exists.", loginUsername);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        // 低优先级用户不能获取高优先级用户信息
        if (logUser.getId() > roleId) {
            log.error("Permission check deny: login user{}, request user{}", logUser.getId(), roleId);
            throw new CommonErrorCodeException(ErrorCode.ERR_PERMISSION);
        }

        User newUser = new User();
        newUser.setUsername(username);
        newUser.setRoleId(roleId);
        newUser.setGender(logUser.getId());
        if (userMapper.insert(newUser, true) != 1) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
        }

        newUser = userDbService.getAccountUserByName(username);

        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        UserCredential        uc      = new UserCredential();
        uc.setCredential(encoder.encode(password));
        uc.setIdentifier(username);
        uc.setIdentityType(1);
        uc.setUserId(newUser.getId());
        uc.setState(CommonStatus.NORMAL.getValue());
        uc.setErrorCnt(0);

        if (userCredentialMapper.insert(uc, true) != 1) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeUser(String uid) {
        User user = userDbService.getAccountUserByUid(uid);

        if (user == null) {
            log.error("Login User({}) is not exists.", uid);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        long minUserId = 1000;
        if (user.getId() < minUserId) {
            log.error("Cant remove system build in user: [{}]", user.getId());
            throw new CommonErrorCodeException(ErrorCode.ERR_PERMISSION);
        }

        if (userMapper.deleteByCondition(USER.ID.eq(user.getId())) != 1) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
        }

        if (userCredentialMapper.deleteById(user.getUserCredential().getId()) != 1) {
            throw new CommonErrorCodeException(ErrorCode.ERR_DATABASE);
        }
    }
}
